@@ -126,9 +126,9 @@ class Dataset(BaseDataset):
            this could lead to poor speed (sometimes stuck)
     """
 
-    data_kinds = {DataKind.FILE, DataKind.MEM_SEQ}
+    data_kinds = {DataKind.FILE, DataKind.MMAP_FILE, DataKind.MEM_SEQ}
 
-    def __init__(self, data, kind=DataKind.MEM_SEQ, transform: Optional[Callable] = None):
+    def __init__(self, data, filename='', kind=DataKind.MEM_SEQ, transform: Optional[Callable] = None):
         """
 
         Args:
@@ -140,7 +140,7 @@ class Dataset(BaseDataset):
         if kind not in self.data_kinds:
             raise ValueError(f'not supported data kind: {kind}, choose one from [{",".join(self.data_kinds)}]')
 
-        self.meta = {'offset': []}
+        self.offset = []
         self.n_data = 0
 
         if self.kind == DataKind.MEM_SEQ:
@@ -152,24 +152,37 @@ class Dataset(BaseDataset):
         elif self.kind == DataKind.FILE:
             if not os.path.exists(data):
                 raise ValueError(f'filename does not exist: {data}')
-
-            logger.debug(f'loading offset from {data}')
-            with open(data, 'rb') as fd:
-                offset = [0]
-                while fd.readline():
-                    offset.append(fd.tell())
-                self.meta['offset'] = offset[:-1]
-            logger.debug(f'loading offset done: n_offset={len(self.meta["offset"])}')
-
-            self.n_data = len(self.meta['offset'])
-
             self.filename = data
+
+            self.offset, self.n_data = self._get_offset(self.filename)
             self.fd = open(data, 'rb', buffering=0)
             self.lock = multiprocessing.Lock()
             logger.warning('with multiprocessing.Lock lead to poor speed')
+        elif self.kind == DataKind.MMAP_FILE:
+            if not os.path.exists(filename):
+                raise ValueError(f'filename does not exist: {filename}')
+            self.filename = filename
+
+            self.offset, self.n_data = self._get_offset(self.filename)
+            self.data = data  # mmap
 
         self.transform = transform
 
+    @staticmethod
+    def _get_offset(filename):
+        logger.debug(f'loading offset from {filename}')
+        with open(filename, 'rb') as fd:
+            offset = [0]
+            while fd.readline():
+                offset.append(fd.tell())
+
+            offset = offset[:-1]
+
+        n_data = len(offset)
+        logger.debug(f'loading offset done: n_offset={n_data}')
+
+        return offset, n_data
+
     def __len__(self) -> int:
         return self.n_data
 
@@ -183,13 +196,23 @@ class Dataset(BaseDataset):
             data = self.data[index]
         elif self.kind == DataKind.FILE:
             with self.lock:
-                self.fd.seek(self.meta['offset'][index])
+                self.fd.seek(self.offset[index])
                 line = self.fd.readline()
 
             try:
                 data = bytes_to_str(line).strip('\n')
             except Exception as e:
-                logger.error(f'decode failed: index={index}, offset={self.meta["offset"][index]}, line={line}')
+                logger.error(f'decode failed: index={index}, offset={self.offset[index]}, line={line}')
+                raise e
+        elif self.kind == DataKind.MMAP_FILE:
+            start = self.offset[index]
+            end = self.offset[index + 1] if index + 1 < self.n_data else self.n_data
+            line = self.data[start:end]
+
+            try:
+                data = bytes_to_str(line).strip('\n')
+            except Exception as e:
+                logger.error(f'decode failed: index={index}, offset={self.offset[index]}, line={line}')
                 raise e
 
         data = apply_transform(self.transform, data)
